@@ -1,7 +1,7 @@
 /*
  * Main application class
  *
- * Copyright (C) 2003,2004,2005  Enrico Zini
+ * Copyright (C) 2003--2010  Enrico Zini <enrico@enricozini.org>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 #include "BuffyWindow.h"
 
 #include "Environment.h"
-#include "buffy/MailProgram.h"
 
 #include <gtkmm/stock.h>
 #include <gtkmm/fileselection.h>
@@ -32,7 +31,6 @@
 #include <map>
 
 using namespace std;
-using namespace stringf;
 using namespace buffy;
 
 
@@ -40,15 +38,18 @@ BuffyWindow::BuffyWindow()
 	: updateButton("Update"),
 	  prefDialog(*this)
 {
-	Config& conf = Environment::get().config();
+	config::Config& conf = Environment::get().config();
 
 	set_title("Buffy Folder List");
 	//set_border_width(5);
-	ConfigItemDictionary prefs = conf.application("buffy");
-	string saved_w = prefs.get("winw", "500").get();
-	string saved_h = prefs.get("winh", "300").get();
+	config::Section prefs = conf.application("buffy");
+	prefs.addDefault("winw", "500");
+	prefs.addDefault("winh", "500");
+
+	int saved_w = prefs.getInt("winw");
+	int saved_h = prefs.getInt("winh");
 	
-	set_default_size(atoi(saved_w.c_str()), atoi(saved_h.c_str()));
+	set_default_size(saved_w, saved_h);
 
 	add(mainVBox);
 
@@ -66,11 +67,11 @@ BuffyWindow::BuffyWindow()
 
 	actionGroup->add(actionRescanFolders = Gtk::Action::create("RescanFolders", "_Update folder list", ""),
 			sigc::mem_fun(*this, &BuffyWindow::rescanFolders));
-	actionGroup->add(actionViewEmpty = Gtk::ToggleAction::create("ViewAll", "View _all folders", "", conf.view_empty().get()),
+	actionGroup->add(actionViewEmpty = Gtk::ToggleAction::create("ViewAll", "View _all folders", "", conf.view().empty()),
 			sigc::mem_fun(*this, &BuffyWindow::on_filter_changed));
-	actionGroup->add(actionViewRead = Gtk::ToggleAction::create("ViewNonEmpty", "View all _non empty folders", "", conf.view_read().get()),
+	actionGroup->add(actionViewRead = Gtk::ToggleAction::create("ViewNonEmpty", "View all _non empty folders", "", conf.view().read()),
 			sigc::mem_fun(*this, &BuffyWindow::on_filter_changed));
-	actionGroup->add(actionViewImportant = Gtk::ToggleAction::create("ViewImportant", "View all folders with _important messages", "", conf.view_important().get()),
+	actionGroup->add(actionViewImportant = Gtk::ToggleAction::create("ViewImportant", "View all folders with _important messages", "", conf.view().important()),
 			sigc::mem_fun(*this, &BuffyWindow::on_filter_changed));
 
 	uimanager = Gtk::UIManager::create();
@@ -101,7 +102,7 @@ BuffyWindow::BuffyWindow()
 	}
 	catch(const Glib::Error& ex)
 	{
-		warning("Building menus failed: %.*s\n", PFSTR(ex.what()));
+		warning("Building menus failed: %s\n", ex.what().c_str());
 	}
 
 	Gtk::Widget* menuBar = uimanager->get_widget("/MenuBar") ;
@@ -130,7 +131,7 @@ BuffyWindow::BuffyWindow()
 	updateButton.signal_clicked().connect(
 			sigc::mem_fun(folderList, &FolderList::do_update));
 
-	set_timer(conf.update_interval().get());
+	set_timer(conf.general().interval());
 
 	Environment::get().signal_child_exit().connect(sigc::mem_fun(*this, &BuffyWindow::on_program_exited));
 
@@ -155,15 +156,15 @@ bool BuffyWindow::on_configure_event(GdkEventConfigure* c)
 	int x, y;
 	Gtk::Window::get_position(x, y);
 
-	Config& conf = Environment::get().config();
-	ConfigItemDictionary prefs = conf.application("buffy");
+	config::Config& conf = Environment::get().config();
+	config::Section prefs = conf.application("buffy");
 
 	//conf.set_application_data("buffy", "winx", fmt(c->x));
 	//conf.set_application_data("buffy", "winy", fmt(c->y));
-	prefs.get("winx", fmt(x)).set(fmt(x));
-	prefs.get("winy", fmt(y)).set(fmt(y));
-	prefs.get("winw", fmt(c->width)).set(fmt(c->width));
-	prefs.get("winh", fmt(c->height)).set(fmt(c->height));
+	prefs.setInt("winx", x);
+	prefs.setInt("winy", y);
+	prefs.setInt("winw", c->width);
+	prefs.setInt("winh", c->height);
 
 	return res;
 }
@@ -172,12 +173,14 @@ void BuffyWindow::on_show()
 {
 	Gtk::Window::on_show();
 
-	Config& conf = Environment::get().config();
-	ConfigItemDictionary prefs = conf.application("buffy");
-	string saved_x = prefs.get("winx", "").get();
-	string saved_y = prefs.get("winy", "").get();
-	if (saved_x != "" && saved_y != "")
-		move(atoi(saved_x.c_str()), atoi(saved_y.c_str()));
+	config::Config& conf = Environment::get().config();
+	config::Section prefs = conf.application("buffy");
+	if (prefs.isSet("winx") && prefs.isSet("winy"))
+	{
+		string saved_x = prefs.get("winx");
+		string saved_y = prefs.get("winy");
+		move(prefs.getInt("winx"), prefs.getInt("winy"));
+	}
 
 	if (folderList.size() == 0)
 		rescanFolders();
@@ -185,17 +188,20 @@ void BuffyWindow::on_show()
 
 void BuffyWindow::on_open_folder(MailFolder folder)
 {
-	debug("OOF activated %.*s\n", PFSTR(folder.path()));
+	debug("OOF activated %s\n", folder.path().c_str());
 
-	MailProgram m = Environment::get().config().mail_programs().selected();
+	config::MailProgram m = Environment::get().config().selectedMailProgram();
+	verbose("Running %s\n", m.command("gui").c_str());
+	m.run(folder, "gui");
+
+	/*
 	//MailProgram m("Mutt", "x-terminal-emulator -e mutt -f %p");
 	if (m)
 	{
-		verbose("Running %.*s\n", PFSTR(m.command()));
-		m.run(folder);
 	}
 	else
 		warning("No mail programs are currently selected.  This is probably due to hand-editing the configuration file and forgetting to add selected=\"true\" to one mail program.\n");
+	*/
 }
 
 void BuffyWindow::on_quit()
@@ -205,7 +211,7 @@ void BuffyWindow::on_quit()
 
 void BuffyWindow::on_filter_changed()
 {
-	Config& conf = Environment::get().config();
+	config::Config& conf = Environment::get().config();
 
 	bool empty = actionViewEmpty->get_active();
 	bool read = actionViewRead->get_active();
@@ -215,9 +221,9 @@ void BuffyWindow::on_filter_changed()
 			read ? "read" : "",
 			important ? "important" : "");
 
-	conf.view_empty().set(empty);
-	conf.view_read().set(read);
-	conf.view_important().set(important);
+	conf.view().setEmpty(empty);
+	conf.view().setRead(read);
+	conf.view().setImportant(important);
 
 	folderList.configureFilter(conf);
 }
